@@ -1,27 +1,22 @@
 package SQL::Dialects::ANSI;
 
-#########################################################################
-#
-# This module is copyright (c), 2001,2005 by Jeff Zucker.
-# This module is copyright (c), 2007-2020 by Jens Rehsack.
-# All rights reserved.
-#
-# It may be freely distributed under the same terms as Perl itself.
-#
-# See below for help (search for SYNOPSIS)
-#########################################################################
 
-use strict;
-use warnings FATAL => "all";
+=head1 NAME
 
-use vars qw($VERSION);
-$VERSION = '1.413_001';
+ SQL::Dialects::ANSI -- ANSI config file for SQL::Parse
 
-use SQL::Dialects::Role;
+=head1 SYNOPSIS
 
-sub get_config
-{
-    return <<EOC;
+  see SQL::Parse, SQL::Squish
+
+=head1 DESCRIPTION
+
+ The makemaker police say i gotta have one of these
+
+=cut
+
+sub get_config {
+return <<EOC;
 [VALID COMMANDS]
 CREATE
 DROP
@@ -310,60 +305,3 @@ ZONE
 EOC
 }
 1;
-
-=pod
-
-=head1 NAME
-
-SQL::Dialects::ANSI
-
-=head1 SYNOPSIS
-
-  use SQL::Dialects::ANSI;
-  $config = SQL::Dialects::ANSI->get_config();
-
-=head1 DESCRIPTION
-
-This package provides the necessary configuration for ANSI SQL, as far it's
-supported.
-
-=head1 FUNCTIONS
-
-=head2 get_config
-
-Returns the configuration for ANSI SQL. The configuration is delivered in
-ini-style:
-
-  [VALID COMMANDS]
-  ...
-
-  [VALID OPTIONS]
-  ...
-
-  [VALID COMPARISON OPERATORS]
-  ...
-
-  [VALID DATA TYPES]
-  ...
-
-  [RESERVED WORDS]
-  ...
-
-=head1 AUTHOR & COPYRIGHT
-
- This module is
-
- copyright (c) 2001,2005 by Jeff Zucker and
- copyright (c) 2007-2020 by Jens Rehsack.
-
- All rights reserved.
-
-The module may be freely distributed under the same terms as
-Perl itself using either the "GPL License" or the "Artistic
-License" as specified in the Perl README file.
-
-Jeff can be reached at: jzuckerATcpan.org
-Jens can be reached at: rehsackATcpan.org or via dbi-devATperl.org
-
-
-=cut
